=== modified file '.bzrignore'
--- .bzrignore	2011-03-10 14:22:12 +0000
+++ .bzrignore	2011-07-10 22:53:59 +0000
@@ -8,3 +8,5 @@
 loggerhead-memprofile
 ./docs/_build/
 tags
+.project
+.pydevproject

=== modified file 'NEWS'
--- NEWS	2011-09-08 00:33:28 +0000
+++ NEWS	2011-11-08 16:29:37 +0000
@@ -42,6 +42,13 @@
     - Fix support for displaying foreign revision ids.
       (Jelmer Vernooij, #736026)
 
+    - Add hook 'controller' to BranchWSGIApp, allowing other bzr plugins
+      to provide custom branch-specific controllers. (Jelmer Vernooij, #585822)
+
+    - Add privacy notification code to loggerhead, allowing branches to be
+      marked as private and a notification ribbon to be displayed via
+      javascript on the view pages. (JC Sackett, #823471)
+
 
 1.18.1 [24Mar2011]
 ------------------

=== modified file 'info.py'
--- info.py	2011-03-19 08:35:57 +0000
+++ info.py	2011-11-23 08:33:12 +0000
@@ -3,11 +3,12 @@
 
 bzr_plugin_name = "loggerhead"
 
-bzr_plugin_version = (1, 18, 0)
+bzr_plugin_version = (1, 18, 1)  # Keep in sync with loggerhead/__init__.py
 
 bzr_compatible_versions = [
     (1, 17, 0), (1, 18, 0), (2, 0, 0), (2, 1, 0), (2, 2, 0), (2, 3, 0),
-    (2, 4, 0)]
+    (2, 4, 0), (2, 5, 0),
+    ]
 
 bzr_minimum_version = bzr_compatible_versions[0]
 

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2011-03-24 03:40:33 +0000
+++ loggerhead/__init__.py	2011-11-23 08:33:12 +0000
@@ -22,7 +22,7 @@
 
 import pkg_resources
 
-__version__ = '1.18.1'
+__version__ = '1.18.1'  # Keep in sync with ../info.py.
 required_bzrlib = (1, 17)
 
 pkg_resources.get_distribution('Paste>=1.6')

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-08-17 11:04:00 +0000
+++ loggerhead/apps/branch.py	2011-11-23 08:18:35 +0000
@@ -13,7 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-#
+
 """The WSGI application for serving a Bazaar branch."""
 
 import logging
@@ -22,6 +22,7 @@
 
 import bzrlib.branch
 import bzrlib.errors
+from bzrlib.hooks import Hooks
 import bzrlib.lru_cache
 
 from paste import request
@@ -33,7 +34,7 @@
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.changelog_ui import ChangeLogUI
 from loggerhead.controllers.diff_ui import DiffUI
-from loggerhead.controllers.download_ui import DownloadUI
+from loggerhead.controllers.download_ui import DownloadUI, DownloadTarballUI
 from loggerhead.controllers.filediff_ui import FileDiffUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
@@ -49,7 +50,13 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT, use_cdn=False):
+                 served_url=_DEFAULT, use_cdn=False, private=False,
+                 export_tarballs=True):
+        """Create branch-publishing WSGI app.
+
+        :param export_tarballs: If true, allow downloading snapshots of revisions
+            as tarballs.
+        """
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -61,6 +68,14 @@
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
+        self.private = private
+        self.export_tarballs = export_tarballs
+
+    def public_private_css(self):
+        if self.private:
+            return "private"
+        else:
+            return "public"
 
     def get_history(self):
         revinfo_disk_cache = None
@@ -123,6 +138,7 @@
         'revision': RevisionUI,
         'search': SearchUI,
         'view': ViewUI,
+        'tarball': DownloadTarballUI,
         }
 
     def last_updated(self):
@@ -155,6 +171,10 @@
                     self.served_url = self.url([])
                 except bzrlib.errors.InvalidURL:
                     self.served_url = None
+        for hook in self.hooks['controller']:
+            controller = hook(self, environ)
+            if controller is not None:
+                return controller
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
@@ -165,9 +185,9 @@
             environ['loggerhead.as_json'] = True
             path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
-        if cls is None:
-            raise httpexceptions.HTTPNotFound()
-        return cls(self, self.get_history)
+        if cls is not None:
+            return cls(self, self.get_history)
+        raise httpexceptions.HTTPNotFound()
 
     def app(self, environ, start_response):
         self.branch.lock_read()
@@ -181,3 +201,23 @@
                 raise
         finally:
             self.branch.unlock()
+
+
+class BranchWSGIAppHooks(Hooks):
+    """A dictionary mapping hook name to a list of callables for WSGI app branch hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+        """
+        Hooks.__init__(self, "bzrlib.plugins.loggerhead.apps.branch",
+            "BranchWSGIApp.hooks")
+        self.add_hook('controller',
+            "Invoked when looking for the controller to use for a "
+            "branch subpage. The api signature is (branch_app, environ)."
+            "If a hook can provide a controller, it should return one, "
+            "as a standard WSGI app. If it can't provide a controller, "
+            "it should return None", (1, 19))
+
+
+BranchWSGIApp.hooks = BranchWSGIAppHooks()

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2010-05-12 14:38:05 +0000
+++ loggerhead/config.py	2011-07-07 19:09:11 +0000
@@ -36,6 +36,7 @@
         use_cdn=False,
         sql_dir=None,
         allow_writes=False,
+        export_tarballs=True,
         )
     parser.add_option("--user-dirs", action="store_true",
                       help="Serve user directories as ~user.")
@@ -75,6 +76,8 @@
                       help="The directory to place the SQL cache in")
     parser.add_option("--allow-writes", action="store_true",
                       help="Allow writing to the Bazaar server.")
+    parser.add_option("--export-tarballs", action="store_true",
+                      help="Allow exporting revisions to tarballs.")
     return parser
 
 

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2011-06-28 16:09:37 +0000
+++ loggerhead/controllers/__init__.py	2011-07-07 19:09:11 +0000
@@ -21,7 +21,7 @@
 import simplejson
 import time
 
-from paste.httpexceptions import HTTPNotFound
+from paste.httpexceptions import HTTPNotFound, HTTPSeeOther
 from paste.request import path_info_pop, parse_querystring
 
 from loggerhead import util

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2010-05-05 19:03:40 +0000
+++ loggerhead/controllers/download_ui.py	2011-11-25 01:41:07 +0000
@@ -19,46 +19,51 @@
 
 import logging
 import mimetypes
+import os
 import urllib
 
 from paste import httpexceptions
 from paste.request import path_info_pop
 
 from loggerhead.controllers import TemplatedBranchView
+from loggerhead.exporter import export_archive
 
 log = logging.getLogger("loggerhead.controllers")
 
 
 class DownloadUI (TemplatedBranchView):
 
-    def __call__(self, environ, start_response):
-        # /download/<rev_id>/<file_id>/[filename]
-
-        h = self._history
-
+    def encode_filename(self, filename):
+
+        return urllib.quote(filename.encode('utf-8'))
+
+    def get_args(self, environ):
         args = []
         while True:
             arg = path_info_pop(environ)
             if arg is None:
                 break
             args.append(arg)
+        return args
 
+    def __call__(self, environ, start_response):
+        # /download/<rev_id>/<file_id>/[filename]
+        h = self._history
+        args = self.get_args(environ)
         if len(args) < 2:
             raise httpexceptions.HTTPMovedPermanently(
                 self._branch.absolute_url('/changes'))
-
         revid = h.fix_revid(args[0])
         file_id = args[1]
         path, filename, content = h.get_file(file_id, revid)
         mime_type, encoding = mimetypes.guess_type(filename)
         if mime_type is None:
             mime_type = 'application/octet-stream'
-
         self.log.info('/download %s @ %s (%d bytes)',
                       path,
                       h.get_revno(revid),
                       len(content))
-        encoded_filename = urllib.quote(filename.encode('utf-8'))
+        encoded_filename = self.encode_filename(filename)
         headers = [
             ('Content-Type', mime_type),
             ('Content-Length', str(len(content))),
@@ -67,3 +72,36 @@
             ]
         start_response('200 OK', headers)
         return [content]
+
+
+class DownloadTarballUI(DownloadUI):
+
+    def __call__(self, environ, start_response):
+        """Stream a tarball from a bazaar branch."""
+        # Tried to re-use code from downloadui, not very successful
+        if not self._branch.export_tarballs:
+            raise httpexceptions.HTTPForbidden(
+                "Tarball downloads are not allowed")
+        archive_format = "tgz"
+        history = self._history
+        self.args = self.get_args(environ)
+        if len(self.args):
+            revid = history.fix_revid(self.args[0])
+            version_part = '-r' + self.args[0]
+        else:
+            revid = self.get_revid()
+            version_part = ''
+        # XXX: Perhaps some better suggestion based on the URL or path?
+        #
+        # TODO: Perhaps set the tarball suggested mtime to the revision
+        # mtime.
+        root = self._branch.friendly_name or 'branch'
+        encoded_filename = self.encode_filename(
+            root + version_part + '.' + archive_format)
+        headers = [
+            ('Content-Type', 'application/octet-stream'),
+            ('Content-Disposition',
+                "attachment; filename*=utf-8''%s" % (encoded_filename,)),
+            ]
+        start_response('200 OK', headers)
+        return export_archive(history, root, revid, archive_format)

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2011-09-02 09:43:57 +0000
+++ loggerhead/controllers/revision_ui.py	2011-11-23 08:18:35 +0000
@@ -140,6 +140,7 @@
                 self._branch.friendly_name,
                 self._branch.is_root,
                 'changes'))
+        can_export = self._branch.export_tarballs
 
         values.update({
             'history': self._history,
@@ -153,6 +154,7 @@
             'filter_file_id': filter_file_id,
             'diff_chunks': diff_chunks,
             'query': query,
+            'can_export': can_export,
             'specific_path': path,
             'start_revid': start_revid,
         })

=== added file 'loggerhead/exporter.py'
--- loggerhead/exporter.py	1970-01-01 00:00:00 +0000
+++ loggerhead/exporter.py	2011-11-24 07:46:41 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+"""Exports an archive from a bazaar branch"""
+
+from bzrlib.export import get_export_generator
+
+
+class ExporterFileObject(object):
+    """Shim that accumulates temporarily written out data.
+
+    There are python tarfile classes that want to write to a file like object.
+    We want to stream data.  But wsgi assumes it can pull data from the
+    handler, rather than having bytes pushed.
+
+    So this class holds the data temporarily, until it is pulled.  It 
+    should never buffer everything because as soon as a chunk is produced, 
+    wsgi will be given the chance to take it.
+    """
+
+    def __init__(self):
+        self._buffer = []
+
+    def write(self, s):
+        self._buffer.append(s)
+
+    def get_buffer(self):
+        try:
+            return ''.join(self._buffer)
+        finally:
+            self._buffer = []
+
+    def close(self):
+        pass
+
+
+def export_archive(history, root, revid, archive_format):
+    """Export tree contents to an archive
+
+    :param history: Instance of history to export
+    :param root: Root location inside the archive.
+    :param revid: Revision to export
+    :param archive_format: Format of the archive, eg 'tar.gz'.
+    """
+    fileobj = ExporterFileObject()
+    tree = history._branch.repository.revision_tree(revid)
+    for _ in get_export_generator(tree=tree, root=root, fileobj=fileobj,
+        format=archive_format):
+        yield fileobj.get_buffer()
+    # Might have additonal contents written
+    yield fileobj.get_buffer()

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2011-08-17 11:04:00 +0000
+++ loggerhead/history.py	2011-11-23 08:29:50 +0000
@@ -33,6 +33,7 @@
 import re
 import textwrap
 import threading
+import tarfile
 
 from bzrlib import tag
 import bzrlib.branch
@@ -803,4 +804,6 @@
             renamed=sorted(reporter.renamed, key=lambda x: x.new_filename),
             removed=sorted(reporter.removed, key=lambda x: x.filename),
             modified=sorted(reporter.modified, key=lambda x: x.filename),
-            text_changes=sorted(reporter.text_changes, key=lambda x: x.filename))
+            text_changes=sorted(reporter.text_changes,
+                                key=lambda x: x.filename))
+

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2011-04-22 17:22:49 +0000
+++ loggerhead/static/css/global.css	2011-11-23 17:21:39 +0000
@@ -343,3 +343,68 @@
 #footer {
     font-size: 77%;
     }
+
+    
+/* ====================
+   Global notifications
+*/
+/* Move the content down so the notification banner doesn't hide any content. */
+body.global-notification-visible {
+    padding-top: 40px;
+    }
+body.global-notification-visible .login-logout {
+    top: 45px;
+    }
+.notification-private {
+    background-image: url(/static/images/notification-private.png); /* sprite-ref: icon-sprites */
+    background-repeat: no-repeat;
+    }
+.global-notification {
+    position: fixed;
+    z-index: 10;
+    top: 0;
+    left: 0;
+    right: 0;
+    padding: 8px 20px;
+    /* Define colour for browsers that don't support transparency */
+    background-color: #8d1f1f;
+    /* Set transparent background for browsers that support it */
+    background-color: rgba(125,0,0,0.9);
+    color: #fff;
+    text-shadow: 0 -1px 0 #631616;
+    font-size: 14px;
+    line-height: 21px;
+    font-weight: bold;
+    -moz-box-shadow: 0 0 5px #333;
+    -webkit-box-shadow: 0 0 5px #333;
+    box-shadow: 0 0 5px #333;
+    }
+.global-notification .sprite.notification-private {
+    float: left;
+    display: inline-block;
+    height: 21px;
+    width: 20px;
+    margin-right: 10px;
+    padding: 0;
+    }
+.global-notification-close, .global-notification-close:active,
+.global-notification-close:visited {
+    color: #e47a7a;
+    }
+.global-notification-close {
+    display: block;
+    position: absolute;
+    top: 11px;
+    right: 20px;
+    font-size: 12px;
+    font-weight: normal;
+    line-height: 14px;
+    }
+.global-notification-close .sprite.notification-close {
+    float: right;
+    display: block;
+    height: 9px;
+    width: 8px;
+    margin: 3px 0 0 7px;
+    padding: 0;
+    }

=== added directory 'loggerhead/static/downloads'
=== added file 'loggerhead/static/images/notification-private.png'
Binary files loggerhead/static/images/notification-private.png and loggerhead/static/images/notification-private.png differ
=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2009-04-01 15:28:21 +0000
+++ loggerhead/static/javascript/custom.js	2011-10-26 16:06:54 +0000
@@ -1,4 +1,4 @@
-Y = YUI().use("node", "io-base", "anim");
+Y = YUI().use("base", "node", "io-base", "anim");
 
 var global_timeout_id = null;
 var global_search_request = null;
@@ -280,3 +280,89 @@
   }
 };
 
+var notification_node = null;
+/*
+ * Display privacy notifications
+ *
+ * This should be called after the page has loaded e.g. on 'domready'.
+ */
+function setup_privacy_notification(config) {
+    if (notification_node !== null) {
+        return;
+    }
+    var notification_text = 'The information on this page is private';
+    var hidden = true;
+    var target_id = "loggerheadCont";
+    if (config !== undefined) {
+        if (config.notification_text !== undefined) {
+            notification_text = config.notification_text;
+        }
+        if (config.hidden !== undefined) {
+            hidden = config.hidden;
+        }
+        if (config.target_id !== undefined) {
+            target_id = config.target_id;
+        }
+    }
+    var id_selector = "#" + target_id;
+    var main = Y.get(id_selector);
+    notification_node = Y.Node.create('<div></div>')
+        .addClass('global-notification');
+    if (hidden) {
+        notification_node.addClass('hidden');
+    }
+    var notification_span = Y.Node.create('<span></span>')
+        .addClass('sprite')
+        .addClass('notification-private');
+    notification_node.set('innerHTML', notification_text);
+    main.appendChild(notification_node);
+    notification_node.appendChild(notification_span);
+};
+
+function display_privacy_notification() {
+    /* Set a temporary class on the body for the feature flag,
+     this is because we have no way to use feature flags in
+     css directly. This should be removed if the feature
+     is accepted. */
+    var body = Y.get('body');
+    body.addClass('feature-flag-bugs-private-notification-enabled');
+    // Set the visible flag so that the content moves down.
+    body.addClass('global-notification-visible');
+
+    setup_privacy_notification();
+    var global_notification = Y.get('.global-notification');
+    if (global_notification.hasClass('hidden')) {
+        global_notification.addClass('transparent');
+        global_notification.removeClass('hidden');
+
+        var fade_in = new Y.Anim({
+            node: global_notification,
+            to: {opacity: 1},
+            duration: 0.3
+        });
+        var body_space = new Y.Anim({
+            node: 'body',
+            to: {'paddingTop': '40px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+        var black_link_space = new Y.Anim({
+            node: '.black-link',
+            to: {'top': '45px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+
+        fade_in.run();
+        body_space.run();
+        black_link_space.run();
+    }
+};
+
+Y.on('domready', function() {
+    var body = Y.get('body');
+    if (body.hasClass('private')) {
+        setup_privacy_notification();
+        display_privacy_notification();
+    }
+});

=== modified file 'loggerhead/templates/branchinfo.pt'
--- loggerhead/templates/branchinfo.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/branchinfo.pt	2011-11-25 01:23:32 +0000
@@ -1,4 +1,5 @@
-<div id="branch-info">
+<div id="branch-info"
+  tal:condition="python:branch.served_url">
   To get this branch, use: <br/>
   <code>bzr branch
   <tal:served-url tal:replace="python:branch.served_url" /></code>

=== modified file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	2008-09-30 02:04:14 +0000
+++ loggerhead/templates/breadcrumbs.pt	2012-01-08 21:43:42 +0000
@@ -10,7 +10,7 @@
 
 <!-- Branch breadcrumbs (for the path within the branch) -->
 <span metal:define-macro="branch" class="breadcrumb">
-    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno, crumb['path']])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
 </span>
 
 <!-- Directory breadcrumbs (for the path leading up to the branch) -->

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2011-03-03 10:58:57 +0000
+++ loggerhead/templates/macros.pt	2011-11-04 18:16:32 +0000
@@ -43,7 +43,7 @@
     <metal:block metal:define-slot="header_extras" />
   </head>
 
-<body>
+ <body tal:attributes="class python:branch.public_private_css()">
     <!-- Loggerhead Content Area -->
       
       <div metal:define-slot="backlink" />

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2011-06-28 16:51:55 +0000
+++ loggerhead/templates/revision.pt	2011-11-23 08:33:55 +0000
@@ -73,7 +73,7 @@
           <li tal:condition="python:remember != change.revno">
             <a tal:attributes="href python:url(['/revision', change.revno], remember=change.revno, compare_revid=None);
                                title string:compare with another revision"
-               tal:content="string:Compare with another revision"></a></li>
+               tal:content="string:compare with another revision"></a></li>
           <li tal:condition="python:(remember is not None) and (compare_revid is None) and (change.revno != remember)" >
             <a tal:attributes="href python:url(['/revision', change.revno], compare_revid=history.get_revno(remember))">
               compare with revision <tal:b content="python:history.get_revno(remember)" />
@@ -84,7 +84,10 @@
                tal:attributes="href python:url(['/diff', change.revno], clear=1)">download diff</a>
             <a tal:condition="python:compare_revid is not None"
                tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">download diff</a>
-          </li>
+           </li>
+           <li tal:condition="python:can_export">
+            <a tal:attributes="href python:url(['/tarball', change.revno])">download tarball</a>
+           </li>
           <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]);
                                            title string:view history from revision ${change/revno}"
                            tal:content="string:view history from revision ${change/revno}"></a></li>

=== added file 'loggerhead/tests/fixtures.py'
--- loggerhead/tests/fixtures.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/fixtures.py	2011-11-25 02:28:56 +0000
@@ -0,0 +1,43 @@
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from __future__ import absolute_import
+
+from fixtures import Fixture
+
+
+class SampleBranch(Fixture):
+
+    def __init__(self, testcase):
+        # Must be a bzr TestCase to hook into branch creation, unfortunately.
+        self.testcase = testcase
+
+    def setUp(self):
+        Fixture.setUp(self)
+
+        self.tree = self.testcase.make_branch_and_tree('.')
+
+        self.filecontents = (
+            'some\nmultiline\ndata\n'
+            'with<htmlspecialchars\n')
+        filenames = ['myfilename', 'anotherfile<']
+        self.testcase.build_tree_contents(
+            (filename, self.filecontents) for filename in filenames)
+        for filename in filenames:
+            self.tree.add(filename, '%s-id' % filename)
+        self.fileid = self.tree.path2id('myfilename')
+        self.msg = 'a very exciting commit message <'
+        self.revid = self.tree.commit(message=self.msg)

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-09-02 09:43:08 +0000
+++ loggerhead/tests/test_controllers.py	2011-11-25 01:41:07 +0000
@@ -1,11 +1,45 @@
+# Copyright (C) 2008-2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import logging
+import tarfile
+import tempfile
+
+from paste.fixture import (
+    AppError,
+    )
+from paste.httpexceptions import HTTPServerError
+
+from testtools.matchers import (
+    Matcher,
+    Mismatch,
+    )
+
+from bzrlib import errors
 import simplejson
 
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.revision_ui import RevisionUI
-from loggerhead.tests.test_simple import BasicTests, consume_app
-from loggerhead import util
+from loggerhead.tests.test_simple import (
+    BasicTests,
+    consume_app,
+    TestWithSimpleTree,
+    )
 
 
 class TestInventoryUI(BasicTests):
@@ -168,6 +202,7 @@
         annotated = annotate_info['annotated']
         self.assertEqual(0, len(annotated))
 
+
 class TestFileDiffUI(BasicTests):
 
     def make_branch_app_for_filediff_ui(self):
@@ -233,3 +268,102 @@
         revlog_ui = branch_app.lookup_app(env)
         self.assertOkJsonResponse(revlog_ui, env)
 
+
+class TestControllerHooks(BasicTests):
+
+    def test_dummy_hook(self):
+        return
+        # A hook that returns None doesn't influence the searching for
+        # a controller.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: None
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertRaises(KeyError, app.lookup_app, env)
+
+    def test_working_hook(self):
+        # A hook can provide an app to use for a particular request.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: "I am hooked"
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertEquals("I am hooked", app.lookup_app(env))
+
+
+class IsTarfile(Matcher):
+
+    def __init__(self, compression):
+        self.compression = compression
+
+    def match(self, content_bytes):
+        f = tempfile.NamedTemporaryFile()
+        try:
+            f.write(content_bytes)
+            f.flush()
+            tarfile.open(f.name, mode='r|' + self.compression)
+        finally:
+            f.close()
+
+
+class MatchesTarballHeaders(Matcher):
+
+    def __init__(self, expect_filename):
+        self.expect_filename = expect_filename
+
+    def match(self, response):
+        # Maybe the c-t should be more specific, but this is probably good for
+        # making sure it gets saved without the client trying to decompress it
+        # or anything.
+        if (response.header('Content-Type') == 'application/octet-stream'
+            and response.header('Content-Disposition') ==
+            "attachment; filename*=utf-8''" + self.expect_filename):
+            pass
+        else:
+            return Mismatch("wrong response headers: %r"
+                % response.headers)
+
+
+class TestDownloadTarballUI(TestWithSimpleTree):
+
+    def setUp(self):
+        super(TestDownloadTarballUI, self).setUp()
+
+    def test_download_tarball(self):
+        # Tarball downloads are enabled by default.
+        app = self.setUpLoggerhead()
+        response = app.get('/tarball')
+        self.assertThat(
+            response.body,
+            IsTarfile('gz'))
+        self.assertThat(
+            response,
+            MatchesTarballHeaders('branch.tgz'))
+
+    def test_download_tarball_of_version(self):
+        app = self.setUpLoggerhead()
+        response = app.get('/tarball/1')
+        self.assertThat(
+            response.body,
+            IsTarfile('gz'))
+        self.assertThat(
+            response,
+            MatchesTarballHeaders('branch-r1.tgz'))
+
+    def test_download_tarball_forbidden(self):
+        app = self.setUpLoggerhead(export_tarballs=False)
+        e = self.assertRaises(
+            AppError, 
+            app.get,
+            '/tarball')
+        self.assertContainsRe(
+            str(e),
+            '(?s).*403 Forbidden'
+            '.*Tarball downloads are not allowed')

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2011-09-08 00:33:28 +0000
+++ loggerhead/tests/test_simple.py	2011-11-25 02:28:56 +0000
@@ -33,6 +33,9 @@
 from paste.fixture import TestApp
 from paste.httpexceptions import HTTPExceptionHandler, HTTPMovedPermanently
 
+from loggerhead.tests.fixtures import (
+    SampleBranch,
+    )
 
 
 class BasicTests(TestCaseWithTransport):
@@ -56,8 +59,8 @@
         self.assertEqual(None, start[2])
         simplejson.loads(content)
 
-    def make_branch_app(self, branch):
-        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name')
+    def make_branch_app(self, branch, **kw):
+        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name', **kw)
         branch_app._environ = {
             'wsgi.url_scheme':'',
             'SERVER_NAME':'',
@@ -71,18 +74,20 @@
 
     def setUp(self):
         BasicTests.setUp(self)
-        self.createBranch()
-
-        self.filecontents = ('some\nmultiline\ndata\n'
-                             'with<htmlspecialchars\n')
-        filenames = ['myfilename', 'anotherfile<']
-        self.build_tree_contents(
-            (filename, self.filecontents) for filename in filenames)
-        for filename in filenames:
-            self.tree.add(filename, '%s-id' % filename)
-        self.fileid = self.tree.path2id('myfilename')
-        self.msg = 'a very exciting commit message <'
-        self.revid = self.tree.commit(message=self.msg)
+        self.sample_branch_fixture = SampleBranch(self)
+
+        # XXX: This could be cleaned up more... -- mbp 2011-11-25
+        self.useFixture(self.sample_branch_fixture)
+        self.tree = self.sample_branch_fixture.tree
+        self.fileid = self.sample_branch_fixture.fileid
+        self.filecontents = self.sample_branch_fixture.filecontents
+        self.msg = self.sample_branch_fixture.msg
+
+    def test_public_private(self):
+        app = self.make_branch_app(self.tree.branch, private=True)
+        self.assertEqual(app.public_private_css(), 'private')
+        app = self.make_branch_app(self.tree.branch)
+        self.assertEqual(app.public_private_css(), 'public')
 
     def test_changes(self):
         app = self.setUpLoggerhead()
@@ -99,7 +104,10 @@
         res = app.get('/changes')
         self.failUnless("To get this branch, use:" in res)
         self.failUnless("lp:loggerhead" in res)
-        app = self.setUpLoggerhead(served_url=None)
+
+    def test_no_empty_download_location(self):
+        """With no served_url, no instructions how to get it"""
+        app = self.setUpLoggerhead()
         res = app.get('/changes')
         self.failIf("To get this branch, use:" in res)
 

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2011-09-08 00:33:28 +0000
+++ loggerhead/util.py	2012-01-08 21:48:52 +0000
@@ -226,7 +226,7 @@
 def html_escape(s):
     """Transform dangerous (X)HTML characters into entities.
 
-    Like cgi.escape, except also escaping " and '. This makes it safe to use
+    Like cgi.escape, except also escaping \" and '. This makes it safe to use
     in both attribute and element content.
 
     If you want to safely fill a format string with escaped values, use
@@ -484,7 +484,7 @@
     for index, dir_name in enumerate(dir_parts):
         inner_breadcrumbs.append({
             'dir_name': dir_name,
-            'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'path': '/'.join(dir_parts[:index + 1]),
             'suffix': '/' + view,
         })
     return inner_breadcrumbs
@@ -555,8 +555,8 @@
 #         for re-ordering an existing page by different sort
 
 t_context = threading.local()
-_valid = ('start_revid', 'file_id', 'filter_file_id', 'q', 'remember',
-          'compare_revid', 'sort')
+_valid = (
+    'start_revid', 'filter_file_id', 'q', 'remember', 'compare_revid', 'sort')
 
 
 def set_context(map):

